///////////////////////////////////////////////////////////
//                     GGBuildTools                      //
//         Gipat Group       //
//             Copyright (C) 2007 Gipat Group            //
//                             //
//    Gipat Group's opened EI-editor-utility license     //
//                       1.0                       //
//                                                       //
//                  www.gipatgroup.org                   //
///////////////////////////////////////////////////////////

//      , ....  :
// 1) Immortal (ImmortalGAD@rambler.ru)
// 2) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////.
//
//   Unit SourcePackService     .
//
////////////////////////////////////////////////////////////////////////.
unit SourcePackService;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, SconfFormat, Archiver, GGConsUtilServiceClass, ExtraFileUtilsLCL,
  Forms, VerInfoTxtFormat, GGBuildToolsShared;
  
const
  GGBT_PackServiceName = 'Source Pack Builder';

type

  //  
  TPackService = class(TGGConsUtilServiceClass)
    protected
      PathToProject :AnsiString;  //    
      SourcePath    :AnsiString;  // ,  
      OutputPath    :AnsiString;  // ,   
      PackageName   :AnsiString;  //  
      ArchiverType  :AnsiString;  //  
      ProjectVer    :AnsiString;  // 
      ProjectRoot   :AnsiString;
      TempFolder    :AnsiString;
      
      
      //   
      function LoadProjectConfiguration(const path :String) : Boolean;
      //    
      procedure ShowOptions; override;
      //   .vit ,  ...
      procedure ResolveProjectVersion;
      
    public
      // 
      constructor Create; override;
      procedure ProcessParameters; override; //     
      procedure DoWork; override;            // ...
      procedure ProcessExtraSconfParams;     //  ,   sconf.
    end;

var
  PackService :TPackService;

implementation

// 
constructor TPackService.Create;
begin
  inherited;
  GGConsAppName := GGBT_PackServiceName;
  //   
end;


//    
procedure TPackService.ShowOptions;
begin
  inherited;
  Writeln('          --filepath=<value> : sets path to the project configuration file.');
  Writeln('Options to redeclare [SourcePackBuilder] section''s elements from project''s');
  Writeln('configuration file:');
  Writeln('          ');
  Writeln('          --FilesDir=<value> : path (relative to project''s root or absolute)');
  Writeln('            to directory with source''s files. All paths may be relative');
  Writeln('            to project''s root or they may be absolute.');
  Writeln('          --OutputDir=<value> : directory, where to put archive with a source.');
  Writeln('          --ArchiveFormart=<value> : format of an archive with a source.');
  Writeln('            Variants: 7z.');
end;

//   .vit ,  ...
procedure TPackService.ResolveProjectVersion;
var
  verFile :TVerInfoTxtFormat;
begin
  //          vit 
  //     -  . ..       
  //    +     ,   '.'  '_';
  
  //       
  // , ,   ,        ,
  //  ,        
  if ProjectVer = 'Project' then begin
    verFile := TVerInfoTxtFormat.Create;
    //    . ,          .sconf,     
    if verFile.Load(ChangeFileExt(PathToProject, '.vit')) then begin
      //ProjectVer := Format('%d_%d_%d_%d', [verFile.Ver1, verFile.Ver2, verFile.Ver3, verFile.Ver4]);
      //    vit  .
      ProjectVer := verFile.GenerateFNVersionString();
    end
    else begin
      //     ,   .
      //   -  1.   , 2.      .
      //        ,     2. 
      // ,    ,       ...
      //WriteLn(verFile.ErrorMessage);
      ProjectVer := '0_0_0_0';
    end;
    verFile.Free;
  end
  else begin
    //   sconf -        .
    StringReplace(ProjectVer, '.', '_', [rfReplaceAll]);
  end;
end;

//   
function TPackService.LoadProjectConfiguration(const path :String) : Boolean;
var
  conf :TSconfFormat;
begin
  //.
  conf := TSconfFormat.Create;
  Result := true;
  
  //  ...
  WriteLn(AddExtensionIfAbsent(path, '.sconf'));
  if (conf.Load(AddExtensionIfAbsent(path, '.sconf'))) then begin
    OutputPath   := conf.SourcePackBuilderSection.OutputDir;
    SourcePath   := conf.SourcePackBuilderSection.FilesDir;
    ArchiverType := conf.SourcePackBuilderSection.ArchiveFormat;
    ProjectVer   := conf.MainSection.ProjectVer;
    PackageName  := GetFileNameWithoutExtension(path);
    ProjectRoot  := conf.MainSection.ProjectRootDir;
  end
  else begin
    //   .
    Result := false;
    Writeln('Corrupted .sconf file format, or wrong path to file...');
  end;
  
  //   .
  conf.Free;
end;

//     
procedure TPackService.ProcessParameters;
begin
  //  
  inherited;
  
  if Application.HasOption('filepath') then begin
    //   .
    Self.PathToProject := Application.GetOptionValue('filepath');
  end;
end;

// ...
procedure TPackService.DoWork;
var
  worker :TArchiver;
  temp   :TTempFolder;
  AllOk : Boolean;
  ExePath : AnsiString;
  
begin
  inherited;
  
  //-.
  AllOk := true;
  ExePath := DelLastSlash(ExtractFilePath(Application.ExeName));   //    .\bin.
  worker := nil;
  
  if (ProcessFurther) then begin
    //  
    ProcessFurther := false;
    
    //....
    if Application.HasOption('filepath') = true then begin
      // .
      ShowVersion;
      WriteLn;
      WriteLn('Loading project Settings...');
      if LoadProjectConfiguration(PathToProject) = false then begin
        //   .
        AllOk := false;
      end;
      
      if AllOk = true then begin
        //  -       ,
        //   sconf.
        Self.ProcessExtraSconfParams;
        //     -  -.
        if Self.ArchiverType = ARCHIVER_NAME_7ZIP then begin
          // ,    .
          worker := T7ZipArchiver.Create;
          //      .
          worker.UtilPath := ResolveLocalPath('..\'+AddLastSlash(GGBT_ExtToolsPath)+GGBT_7zBinName,ExePath);
        end
        else begin
          //-   -  .   .
          AllOk := false;
          Writeln('Unknown archivator type: '+Self.ArchiverType);
        end;
      end;
      
      //   .
      if AllOk = true then begin
        //    ...
        ResolveProjectVersion;
        worker.Name := PackageName + '_' + ProjectVer + '_src';
        temp := TTempFolder.Create(worker.Name);
        worker.OutputDir := ResolveLocalPath(OutputPath, ProjectRoot);
        worker.SourceDir := temp.GetTempPath;
        //    .
        Writeln('Creating temporary source distro '+worker.Name);
        CopyDirectory(ResolveLocalPath(SourcePath, ProjectRoot), temp.GetTempPath, true);
        Writeln('done.');
        Writeln('Creating archive '+worker.GetFullArchiveName());
        if worker.Pack = true then begin
          //.
          Writeln('done.');
        end
        else begin
          Writeln('Error, while creating archive:');
          Writeln(worker.ErrorMessage);
          AllOk := false;
        end;
        
        temp.Free;
      end;

      //   .
      if worker <> nil then begin
        worker.Free;
      end;
    end
    else begin
      //.       .
      Self.ShowHelp();
      Writeln('Error!!! "--filepath=<value>" option is required!');
      Writeln('');
    end;
  end;
end;

procedure TPackService.ProcessExtraSconfParams;
// ,   sconf.
begin

  if Application.HasOption('FilesDir') then begin
    Self.SourcePath := Application.GetOptionValue('FilesDir');
  end;
  
  if Application.HasOption('OutputDir') then begin
    Self.OutputPath := Application.GetOptionValue('OutputDir');
  end;
  
  if Application.HasOption('ArchiveFormart') then begin
    Self.ArchiverType := Application.GetOptionValue('ArchiveFormart');
  end;
  
end;

initialization
  PackService := TPackService.Create;

finalization
  PackService.Free;


end.

